#include <ida.hpp>
#include <idp.hpp>
#include <name.hpp>
#include <loader.hpp>
#include <expr.hpp>
#include <ints.hpp>
#include "Python.h" 

//-------------------------------------------------------------------------
static PyObject *py_getpredef_insn_cmt(PyObject * /*self*/, PyObject *args)
{
  do 
  {
    // Parse arguments
    unsigned long ea;
    if (!PyArg_ParseTuple(args, "k", &ea))
      break;

    // Decode instruction
    if (decode_insn(get_screen_ea()) == 0)
      break;

    // Get comments
    char buf[MAXSTR], *p = buf;
    p += qsnprintf(buf, sizeof(buf), "%s: ", ph.instruc[cmd.itype].name);

    if (get_predef_insn_cmt(cmd, p, sizeof(buf) - (p - buf)) == -1 || *p == '\0')
      break;

    // Return comment as a string
    return PyString_FromString(buf);
  } while (false);
  Py_RETURN_NONE;
}

//-------------------------------------------------------------------------
static PyMethodDef py_methods[] =
{
  {"getpredef_insn_cmt",  py_getpredef_insn_cmt, METH_VARARGS, ""},
  {NULL, NULL, 0, NULL}
};

//-------------------------------------------------------------------------
static const char idc_getenv_args[] = { VT_STR2, 0 };
static const char idc_getenv_name[] = "getenv";

static error_t idaapi idc_getenv(idc_value_t *argv, idc_value_t *res)
{
  char *env = getenv(argv[0].c_str());
  res->set_string(env == NULL ? "" : env);
  return eOk;
}

//--------------------------------------------------------------------------
int idaapi init()
{
  if (!Py_IsInitialized())
    Py_Initialize();

  Py_InitModule("idapyext", py_methods);
  set_idc_func(idc_getenv_name, idc_getenv, idc_getenv_args );
  return PLUGIN_KEEP;
}

//--------------------------------------------------------------------------
void idaapi term(void)
{
  // Unregister
  set_idc_func(idc_getenv_name, NULL, NULL);
}

//--------------------------------------------------------------------------
void idaapi run(int a)
{
}

//--------------------------------------------------------------------------
plugin_t PLUGIN =
{
  IDP_INTERFACE_VERSION,
  // plugin flags: load as early as possible and hide from the plugins menu
  PLUGIN_FIX | PLUGIN_HIDE,
  init,                 // initialize
  term,                 // terminate. this pointer may be NULL.
  run,                  // invoke plugin
  NULL,                 // long comment about the plugin
  NULL,                 // multiline help about the plugin
  "idcpyext",           // the preferred short name of the plugin
  NULL                  // the preferred hotkey to run the plugin
};
